#include <stdio.h>
#include <proto.h>

#include "gtbase.h"

/************************************************************************\
*
* GtBase class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Setup empty object.
*
\************************************************************************/
GtBase::GtBase(void)
{
	del=FALSE;
}

/************************************************************************\
*
* Really destroy object.
*
\************************************************************************/
void GtBase::Destroy(void)
{
	del=TRUE;
}

/************************************************************************\
*
* Try to destroy object.
*
\************************************************************************/
void GtBase::DeInit(void)
{
	this->Destroy();
}

/************************************************************************\
*
* Setup object.
*
\************************************************************************/
sbool GtBase::InitV(ulong tag,...)
{
	int p;
	va_list va;
	sbool res;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	res=this->Init(GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(res);
}

/************************************************************************\
*
* Set object attributes.
*
\************************************************************************/
sbool GtBase::SetV(ulong tag,...)
{
	int p;
	va_list va;
	sbool res;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	res=this->Set(GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(res);
}

/************************************************************************\
*
* Get object attributes.
*
\************************************************************************/
void GtBase::GetV(ulong tag,...)
{
	int p;
	va_list va;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	this->Get(GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
}

/************************************************************************\
*
* Check if object is (sub)class of a named class.
*
\************************************************************************/
sbool GtBase::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtBase")==0)
		return(TRUE);
	else
		return(FALSE);
}

/************************************************************************\
*
* Print error header for any object with it's class.
*
\************************************************************************/
void GtBase::PrtErrHeader(void)
{
	fprintf(stderr,"Gt error in %s:\n ",this->GetClass());
}

/************************************************************************\
*
* Get one attribute and return it (not very effective).
*
\************************************************************************/
long GtBase::GetA(ulong tag,long def)
{
	GtTag ts[2];
	long val;
	
	val=def;
	ts[0].tag=tag;
	ts[0].d.ptr=&val;
	ts[1].tag=GT_DONE;
	this->Get(ts);
	return(val);
}

/************************************************************************\
*
* Destroy any Gt object.
*
\************************************************************************/
void GtDestroy(GtBase *obj)
{
	obj->DeInit();
	if(obj->del)
		delete obj;
}
