#include <stdio.h>

#include "gtall.h"
#include "buttons.h"
#include "labels.h"

/************************************************************************\
*
* Demo program in multiple parts.
*
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

static GtAppShell *shell;
static GtTextView *txt;
static GtScrollBar *hscr,*vscr;
static GtFont *titlefont;

static char *text[]=
{
	"\title",
	"Welcome to the Gt Demo",
	"This is a simple demo of some of the widgets",
	"avaliable in the Gt library. The demo is divided",
	"in a number of parts with each part showing widgets",
	"of a certain category.",
	"\title",
	"How to Use This Demo",
	"This widget is a text view and it will show a",
	"description of the last activated part.",
	"To scroll through this text just drag the scrollers",
	"to the right or below by pressing the left mouse",
	"button on the knob and move it (up/down or left/right).",
	"To activate a part select from the parts menu by",
	"pressing and holding down the right mouse button in",
	"the menubar on top of one of the menu names. A menu",
	"will be pulled down and a menu option may be selected",
	"by moving the mouse over it and releasing the right",
	"mouse button, or by pressing and releasing the left",
	"mouse button while still holding the right mouse button",
	"pressed.",
	NULL
};

/************************************************************************\
*
* Set text in text view.
*
\************************************************************************/
static void SetText(char **t)
{
	int n;
	ulong l,p;
	
	txt->RemAllLines();
	for(n = 0; t[n]; n++)
	{
		if(strcmp(t[n],"\title")==0)
		{
			n++;
			txt->AddLine(GTP_LAST);
			l = txt->AddLine(GTP_LAST);
			p = txt->AddPart(l,t[n],strlen(t[n]),GTP_LAST);
			txt->SetTextAttr(p,titlefont,GTTV_ULINE,0,0);
		}
		else
		{
			l = txt->AddLine(GTP_LAST);
			if(strcmp(t[n],"\nl")!=0)
				p = txt->AddPart(l,t[n],strlen(t[n]),GTP_LAST);
		}
	}
	txt->Refresh();
}

/************************************************************************\
*
* Show text about program.
*
\************************************************************************/
static int AboutCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	obj = obj;
	cd = cd;
	ud = ud;
	SetText(text);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Close main shell and exit program.
*
\************************************************************************/
static int CloseCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtApp *app;
	
	obj = obj;
	cd = cd;
	app = (GtApp *)ud;
	app->BreakLoop();
	return(GTCB_CONT);
}

/************************************************************************\
*
* Size of textview changed.
*
\************************************************************************/
static int TxtChangeCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtTextViewCD *tcd;
	
	obj = obj;
	tcd = (GtTextViewCD *)cd;
	ud = ud;
	hscr->SetV(
		GT_Value,tcd->txtxp,
		GT_End,tcd->txtxs,
		GT_Page,tcd->xsize,
		GT_DONE);
	vscr->SetV(
		GT_Value,tcd->txtyp,
		GT_End,tcd->txtys,
		GT_Page,tcd->ysize,
		GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Value of horizontal scroller changed.
*
\************************************************************************/
static int HorizChangeCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtScrollerCD *scd;
	int xp,yp;
	
	obj = obj;
	scd = (GtScrollerCD *)cd;
	ud = ud;
	txt->GetPosition(&xp,&yp);
	txt->SetPosition(scd->value,yp);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Value of vertical scroller changed.
*
\************************************************************************/
static int VertChangeCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtScrollerCD *scd;
	int xp,yp;
	
	obj = obj;
	scd = (GtScrollerCD *)cd;
	ud = ud;
	txt->GetPosition(&xp,&yp);
	txt->SetPosition(xp,scd->value);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Show labels part.
*
\************************************************************************/
static int LabelsCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	obj = obj;
	cd = cd;
	ud = ud;
	CreateLabels(shell);
	SetText(GetLabelsText());
	return(GTCB_CONT);
}

/************************************************************************\
*
* Show buttons part.
*
\************************************************************************/
static int ButtonsCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	obj = obj;
	cd = cd;
	ud = ud;
	CreateButtons(shell);
	SetText(GetButtonsText());
	return(GTCB_CONT);
}

/************************************************************************\
*
* Create menus of the main shell.
*
\************************************************************************/
static void CreateMenus(GtApp *app,GtMenuBar *parent)
{
	GtMenu *m;
	GtGroup *g;
	GtButton *b;
	GtCascade *c;
	
	m = GtMkMenu("filemenu",parent,NULL);
	g = GtMkGroup("filegrp",m,
		GT_FrameType,GTF_OUT,
		GT_Orient,GTO_VERT,
		GT_DONE);
	
	b = GtMkButton("babout",g,NULL);
	b->AddCB(GT_Activate,AboutCB,NULL);
	GtMkLabel("about",b,NULL);

	GtMkSeparator("sep",g,NULL);

	b = GtMkButton("bquit",g,NULL);
	b->AddCB(GT_Activate,CloseCB,app);
	GtMkLabel("quit",b,NULL);
	
	c = GtMkCascade("filecas",parent,
		GT_Weight,0,
		GT_Menu,m,
		GT_DONE);
	GtMkLabel("file",c,NULL);

	m = GtMkMenu("partmenu",parent,NULL);
	g = GtMkGroup("partgrp",m,
		GT_FrameType,GTF_OUT,
		GT_Orient,GTO_VERT,
		GT_DONE);

	b = GtMkButton("blabels",g,NULL);
	b->AddCB(GT_Activate,LabelsCB,NULL);
	GtMkLabel("labels",b,NULL);
	
	b = GtMkButton("bbuttons",g,NULL);
	b->AddCB(GT_Activate,ButtonsCB,NULL);
	GtMkLabel("buttons",b,NULL);

	c = GtMkCascade("partcas",parent,
		GT_Weight,0,
		GT_Menu,m,
		GT_DONE);
	GtMkLabel("part",c,NULL);
}

/************************************************************************\
*
* Main demo program.
*
\************************************************************************/
int main(int argc,char **argv)
{
	GtApp *app;
	GtAppDisp *dsp;
	GtGroup *g;
	GtMatrix *m;
	GtFrame *f;
	GtMenuBar *mbar;
	int xi,yi;
	char *fname="fixed";
	
	app = GtMkApp(GT_AppName,"demo",GT_AppClass,"Demo",GT_DONE);
	app->SetupRes(&argc,argv);
	
	dsp = GtMkAppDisp(app,NULL);
	
	shell = GtMkAppShell("shell",dsp,NULL);
	shell->AddCB(GT_Close,CloseCB,app);
	
	g = GtMkGroup("vert",shell,
		GT_Orient,GTO_VERT,
		GT_DONE);
	
	mbar = GtMkMenuBar("menubar",g,
		GT_Weight,0,
		GT_DONE);
	
	CreateMenus(app,mbar);
	
	m = GtMkMatrix("matrix",g,
		GT_Columns,2,
		GT_Rows,2,
		GT_DONE);
	
	f = GtMkFrame("txtfrm",m,NULL);
	
	txt = GtMkTextView("text",f,NULL);
	txt->GetTextInc(&xi,&yi);
	txt->AddCB(GT_ValueChange,TxtChangeCB,NULL);
	
	vscr = GtMkScrollBar("vertscr",m,
		GT_Orient,GTO_VERT,
		GT_Start,0,
		GT_Step,yi,
		GT_ReduceInterval,TRUE,
		GT_ReportDrag,TRUE,
		GT_DONE);
	vscr->AddCB(GT_ValueChange,VertChangeCB,NULL);
	
	hscr = GtMkScrollBar("horizscr",m,
		GT_Orient,GTO_HORIZ,
		GT_Start,0,
		GT_Step,xi,
		GT_ReduceInterval,TRUE,
		GT_ReportDrag,TRUE,
		GT_DONE);
	hscr->AddCB(GT_ValueChange,HorizChangeCB,NULL);
	
	app->GetRes(NULL,"titlefontlist",GTRT_STRING,&fname);
	titlefont = GtMkFont(dsp,GT_FontList,fname,GT_DONE);
	
	SetText(text);
	
	shell->Build();
	
	app->EventLoop(GTAM_LOOP);
}
