#ifndef INC_GTAPP_H
#define INC_GTAPP_H

#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/Xresource.h>

#include "gtbase.h"
#include "gtcblist.h"
#include "gtdlink.h"
#include "gtslink.h"

/************************************************************************\
*
* GtApp class header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Modes of EventLoop */
#define GTAM_POLL		0
#define GTAM_WAIT		1
#define GTAM_LOOP		2

/* File descriptor watch modes */
#define GTAWM_READ		0x0001
#define GTAWM_WRITE		0x0002
#define GTAWM_EXCEPT	0x0004

/* File descriptor watch callback data */
typedef struct GtWatchCD
{
	int fd;
	uword mode;
} GtWatchCD;

/* Resource converter callback data */
typedef struct GtResourceCD
{
	int type;
	void *data;
	XrmValue res;
} GtResourceCD;

class GtAppDisp;
class GtWidget;

class GtApp : public GtBase
{
public:
		/* Standard */
	GtApp(void);
	void Destroy(void);
	sbool Init(GtTag *tgs);
	sbool Set(GtTag *tgs);
	void Get(GtTag *tgs);
	char *GetClass(void) { return("GtApp"); }
	sbool IsSubClassOf(char *cn);
		/* Resources */
	void SetupRes(int *ac,char **av);
	sbool LoadRes(char *name,sbool override);
	sbool GetRes(GtWidget *obj,char *name,int type,void *data);
	void AddRConverter(int t,GtCallB cb,GtUDPtr ud);
	void RemRConverter(int t,GtCallB cb);
	int AddRType(char *tname,sbool create);
		/* TimeOut */
	void AddTimeOut(long msec,GtCallB cb,GtUDPtr ud);
	void RemTimeOut(GtCallB cb);
		/* External filedescriptors */
	void AddFDWatch(int fd,uword mode,GtCallB cb,GtUDPtr ud);
	void RemFDWatch(int fd,GtCallB cb);
		/* Loop */
	void BreakLoop(void);
	sbool EventLoop(int mode);
		/* AppDisp linking */
	void AddDisp(GtAppDisp *d);
	void RemDisp(GtAppDisp *d);
		/* Misc */
	char *GtDesc(void);
	void GetArgs(int *ac,char ***av) { *ac=argc; *av=argv; }
protected:
	GtDLink tohd,rchd,ftohd,rthd,fdhd;
	XrmDatabase appdb;
	GtResourceCD rval;
	GtSLink disps;
	GtWidget *robj;
	char *aname,*cname;
	char *nprefix,*cprefix;
	int nplen,cplen,npmlen,cpmlen;
	int argc;
	int nxtrtype;
	char **argv;
	fd_set mrd,mwr,mex;
	ubyte flags;
private:
	char *Prefix(GtWidget *obj,int size,sbool cls);
	void MaxSleepTime(struct timeval *res,struct timeval *x,struct timeval *y);
	void CheckTimeOut(void);
};

extern void GtAppTimerSig(int sig);
extern GtApp *GtMkApp(GtTag *tgs);
extern GtApp *GtMkApp(ulong tag,...);

#endif /* INC_GTAPP_H */
